package AutoupdateDeploy;
use 5.006;
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(runExternalScript restartECS restartTomcat stopVIS restartVIS restartService initialize shutdown writeLog processReturnCode md5sum);

#!/usr/bin/perl -w

use strict;
use Digest::MD5;

=pod

=head1 NAME

AutoupdateUtil
Version: 1.0

=head1 DESCRIPTION

Utility functions used by the autoupdater itself. 

=head2 Methods

=over

=item writeLog($)

Write a timestamped message to the autoupdate-deploy log. The main autoupdate-deploy_v2 script routes all output
from numbered scripts to this function. Numbered scripts should not use writeLog() but should use print() instead
or errors will result.

=item runExternalScript($)

A simple function that checks to see if a script exists and if it does, executes it. The stdout of the process is 
funnelled into the writeLog function. The system exit code is returned.

=item restartECS()

Restarts the event collection service.

=item stopVIS()

Stops the VA scanner service.

=item restartVIS($)

Restarts the VA scanner service.

=item restartTomcat()

Restarts the tomcat service.

=item restartService($)

Restarts the service specified in the passed-in parameter.

=item md5sum($)

Returns the md5 sum for a file

=back

=cut

my $logfileName = "/tmp/autoupdate-deploy.log";
my $LOGFILE;
my %monthMap = (
	"01" => "Jan",
	"02" => "Feb",
	"03" => "Mar",
	"04" => "Apr",
	"05" => "May",
	"06" => "Jun",
	"07" => "Jul",
	"08" => "Aug",
	"09" => "Sep",
	"10" => "Oct",
	"11" => "Nov",
	"12" => "Dec"
);

sub initialize()
{
	open $LOGFILE, ">>", $logfileName;

	AutoupdateDeploy::writeLog("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
	writeLog("Starting autoupdate deploy.");
	AutoupdateDeploy::writeLog("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
}

sub shutdown()
{
	AutoupdateDeploy::writeLog("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
	writeLog("Autoupdate deploy complete.");
	AutoupdateDeploy::writeLog("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");

	close $LOGFILE;
}

sub writeLog($)
{
	(my $logEntry) = @_;
	(my $sec,my $min,my $hour,my $mday,my $mon,my $year,my $wday,my $yday,my $isdst)=localtime(time);
	my $msg = sprintf("%s %02d %02d:%02d:%02d",$monthMap{sprintf("%02d",$mon+1)},$mday,$hour,$min,$sec);
	$msg = $msg = $msg." - ".$logEntry."\n";

	print $LOGFILE $msg;
}

sub runExternalScript($)
{
	(my $scriptName) = @_;

	# Execute it if we find it.
	if(-e $scriptName)
	{
		#my $LOG;
		my $lineout;

		writeLog("Found a script to run: $scriptName");
		unless( open(LOG, "$scriptName"." 2>&1 |") ) { writeLog( "unable to launch script: $!"); return "$!;;"; }
		while(!eof(LOG))
		{
			$lineout = <LOG>;
			writeLog($lineout);
		}
		# The last line should  be the return code string from the script.
		return $lineout;
	}

	return "-1;;";
}

sub restartECS()
{
	restartService("ecs");
}

sub restartTomcat()
{
	restartService("tomcat");
}

sub findVIS()
{
	my $vis = "";
	my $nvaHostConf = $ENV{'NVA_CONF'};
	if ($nvaHostConf !~ /^.*\.hostcontext\.conf$/)
	{
		$nvaHostConf =~ m/^(.*?)\.(.*?)$/;
		$nvaHostConf = $1.".hostcontext.".$2;
	}
	my $COMPONENT_PROCESSES =`grep "COMPONENT_PROCESSES=" $nvaHostConf | cut -d= -f2`;
	chomp ($COMPONENT_PROCESSES);
	if($COMPONENT_PROCESSES =~ m/^.*,vis\.(.*?),.*$/)
	{
		$vis = $1;
	}
	else
	{
		print "vis not found in $COMPONENT_PROCESSES\n";
		print "vis not found in $nvaHostConf\n";
	}
	return $vis;	
}

sub stopVIS()
{
	my $vis = findVIS();
	if($vis ne "")
	{
		my $arg_status = "/opt/qradar/init/vis status $vis";
	my $out = system($arg_status);
	if($out == 0)
	{
		    my $arg_stop = "/opt/qradar/init/vis stop $vis";
	    system($arg_stop);
		}
		my $slept = 0;
		while ($out == 0 && $slept < 30)
		{
			$slept = $slept + sleep(5);
			$out = system($arg_status);
		}
	}
	else
	{
		print "vis not found";
	}
}

sub restartVIS($)
{
	(my $stoppedVIS) = @_;
	my $vis = findVIS();
	if($vis ne "")
	{
		my $arg_status = "/opt/qradar/init/vis status ".$vis;
		my $out = system($arg_status);
		if($out == 0)
		{
		    writeLog("$vis is running... restarting it ...");
		    my $arg_restart = "/opt/qradar/init/vis restart ".$vis;
		    system($arg_restart);
		}
		elsif ($stoppedVIS == 1)
		{
		    writeLog("$vis was stopped in deployment... restarting it ...");
		    my $arg_start = "/opt/qradar/init/vis start ".$vis;
		    system($arg_start);
		}
	}
	else
	{
		writeLog("vis not restarted");
	}
}

sub restartService($)
{
	(my $servicename) = @_;
	
	my $margs = "service $servicename status";
	my $out = system($margs);
	if($out == 0)
	{
	    writeLog("$servicename is running... restarting it ...");
	    my $arg_stop = "service $servicename restart";
	    system($arg_stop);
	}
}

sub processReturnCode($)
{
	(my $line) = @_;
	my @errorcodes;
	my %results;
	
	@errorcodes = split(";;", $line);
	
	foreach my $code (@errorcodes)
	{
		$code =~ m/(.*?)=(.*?)$|(-?\d{1,6})/;
		if(defined($1))
		{
			if(defined($2))
			{
				$results{$1} = $2;
			}
			else
			{
				$results{"errno"} = $1;
			}
		}
	}
	
	return %results;
}

sub md5sum($)
{
	(my $file) = @_;
	my $digest = "";
	if (-e $file)
	{
		eval{
			open(FILE, $file) or die "file $file not found!\n";
			my $ctx = Digest::MD5->new;
			$digest = $ctx->addfile(*FILE);
			$digest = $ctx->hexdigest;
			close(FILE);
		};
	}
	return $digest;
}


