#!/usr/bin/perl -w

#===========================================================================#
# This script replaces bash script /opt/qradar/bin/estreamer-cert-import.sh #
#===========================================================================#

use strict;

use Getopt::Std;
use File::Copy;

my $QRADAR_CONF = '/opt/qradar/conf';
my $PREFIX = 'estreamer';

my $ver = "1.0";

my %options = 
(
	d => 0,
	h => 0,
	f => '',
	o => 'estreamer',
	p => '',
	v => 0
);

getopts('dhf:o:p:v',\%options);

my $prefix = $options{o};


#=========================SUBS===========================#

sub exitWithMessage($$) {
	my ($message,$returncode) = @_;

	print "$message\n\n";
	exit($returncode);
}

sub usage() 
{
	my $usage = <<"ENDOFUSAGE";

Script Version: $ver

Usage: $0 -f <filename>.pkcs12 [options]


	-d	Debug mode.
		Extra output logging around backup of files, output of keystore and truststore
		files, import, and other detailed info.

	-f	Input pkcs12 certificate file - REQUIRED
	
	-h	Help - displays this message

	-o	Output certificate filename prefix override. 
		By default the keystore and truststore filenames are estreamer.keystore and
		estreamer.truststore, specifying this option will change the prefix of the
		filename to the one specified. This is only required when using multiple 
		Estreamer devices to ensure they get unique certificate files.

	-p	Specify password for source certificate decryption.
		When configured according to documentation the pkcs12 file should not have an 
		import password, however if your pkcs12 file has been created with one, use 
		this option to specify the password used during client certificate creation.
		
	-v	Output script version


ENDOFUSAGE

	exitWithMessage($usage,0);
}


sub backupfile($)
{
	my $filename = shift;

	if (-e $filename)
	{
		print "DEBUG: Creating backup of [$filename] before creating new file.\n" if ($options{d});

		if (!move($filename, $filename.".old")) 
		{
			exitWithMessage ("ERROR: Error backing up [$filename]. Aborting import.",1);
		} 
		else 
		{
			print "DEBUG: Backup of [$filename] to [$filename.old] complete.\n" if ($options{d});
		}
	}
}


sub copy_to_trusted_certificates($)
{
	my $filename = shift;

	my $destination = "$QRADAR_CONF/trusted_certificates";

	print "DEBUG: Checking if [$destination] exists.\n" if ($options{d});
	if (-e $destination)
	{
		print "DEBUG: Copying [$filename] to [$QRADAR_CONF/trusted_certificates].\n" if ($options{d});
		
		if (copy($filename,$destination))
		{
			print "DEBUG: Copied [$filename] to [$destination].\n\n" if ($options{d});
		}
		else
		{
			exitWithMessage("ERROR: Failed to copy [$QRADAR_CONF/$filename] to [$destination].\n",1);
		}
	}
	else
	{
			print "DEBUG: Directory [$destination] does not exist, skipping file copy.\n" if ($options{d});
	}
}



#================================= CHECK AND SET INPUT PARAMETERS ========================================#
if ($options{v})
{
	print ("Version: $ver\n");	
}

if ($options{h}) 
{
	usage();
}

if (!$options{f}) 
{
	exitWithMessage("ERROR: Input pkcs12 certificate filename not specified: -f option is mandatory. Please see $0 -h for usage.",1);
}

if (!-e $options{f})
{
	exitWithMessage("ERROR: Input pkcs12 certificate file [$options{f}] was not found.",1);
}



#============================================= MAIN ======================================================#
print "DEBUG: $0 Version $ver\n" if ($options{d});

my $pem_data = `openssl pkcs12 -in $options{f} -nodes -passin pass:$options{p} 2>&1`;


#See if we decrypted the pkcs12 file properly to be able to read the Certificate Data
if ($pem_data !~ /Bag Attributes/) 
{
	#Check for bad password message
	if ($pem_data =~ /invalid password/)
	{
		exitWithMessage("ERROR: Invalid password converting pkcs12 file [$options{f}].\nIf the certificate was created with a password please use the -p option and specify the password used when creating the client certificate on the Estreamer device.\nSee $0 -h for more details.",1);
	}
	else
	{
		#Unknown decoding error
		exitWithMessage("ERROR: Error converting pkcs12 file [$options{f}].",1);
	}
} 
else 
{
	#Try and locate the Server key from the decoded PEM data
	if ($pem_data =~ /(Bag Attributes: <No Attributes>.*?-----END CERTIFICATE-----)/s) 
	{

		#================IMPORT TRUSTSTORE=================#
		open (OUTFILE, '>DefenseCenter.pem') || exitWithMessage("ERROR: Failed to create temporary file [DefenseCenter.pem] - aborting.",1);
		print OUTFILE $1;
		close OUTFILE;

		#Backup the old truststore file by the same name (if it exists)
		backupfile("$QRADAR_CONF/$prefix.truststore");

		my $output;
		$output = `openssl x509 -in DefenseCenter.pem -inform PEM -out DefenseCenter.pem.der -outform DER 2>&1`;
		print "DEBUG: $output\n" if ($options{d});
	
		if (-e 'DefenseCenter.pem.der') 
		{
			unlink 'DefenseCenter.pem' || print "ERROR: Failed to remove temporary file [DefenseCenter.pem].\n";
			$output = "Truststore import: ".`keytool -import -alias DefenseCenter.pem -file DefenseCenter.pem.der -keystore $QRADAR_CONF/$prefix.truststore -storepass estreamer -noprompt 2>&1`."\n";
			print "DEBUG: $output\n" if ($options{d});
			unlink 'DefenseCenter.pem.der' || print "ERROR: Failed to remove temporary file [DefenseCenter.pem.der].\n";

			if ($output !~ /Certificate was added to keystore/)
			{
				exitWithMessage("ERROR: Problem creating truststore certificate.\n",1);
			}
			elsif (!-e "$QRADAR_CONF/$prefix.truststore")
			{
				exitWithMessage("ERROR: Failed to create truststore file [$QRADAR_CONF/$prefix.truststore]. Please ensure the current user has write access to [$QRADAR_CONF].",1);
			}
			else
			{
				print "Successfully generated truststore file [$QRADAR_CONF/$prefix.truststore].\n";
			}
		}
		else
		{
			exitWithMessage("ERROR: Problem importing truststore file. Temporary file [DefenseCenter.pem] was not found.", 1);
		}
			

		#==================IMPORT KEYSTORE=================#

		my $input_pkcs12 = $options{f};

		#We must repackage the pkcs12 with the default password before import
		print "DEBUG: PKCS12 file [$options{f}] is being repackaging to [Client.pkcs12] with default password of [estreamer] for keystore import.\n" if ($options{d});

		#Create a new PEM file
		$output = `openssl pkcs12 -in $input_pkcs12 -out Client.pem -nodes -password pass:$options{p} 2>&1`;
		print "DEBUG: $output\n" if ($options{d});

		if (-e 'Client.pem')
		{
			#Package that PEM back to PKCS12 with the default password
			$output = `openssl pkcs12 -export -in Client.pem -out Client.pkcs12 -passout pass:estreamer 2>&1`;
			print "DEBUG: $output\n" if ($options{d});

			if (-e 'Client.pkcs12')
			{
				unlink('Client.pem') || print "ERROR: Failed to remove temporary file [Client.pkcs12].\n";
		
				$input_pkcs12 = 'Client.pkcs12';
				$options{p} = 'estreamer';
			}
			else
			{
				exitWithMessage("ERROR: Problem importing keystore file. Repackaged pcks12 file [Client.pkcs12] was not found.",1);
			}
		}
		else
		{
			exitWithMessage("ERROR: Problem importing keystore file. Temporary file [Client.pem] was not found.", 1);
		}
		
		#Backup the old keystore file by the same name (if it exists)
		backupfile("$QRADAR_CONF/$prefix.keystore");


		$output = `keytool -importkeystore -srckeystore $input_pkcs12 -srcstorepass estreamer -srcstoretype pkcs12 -destkeystore $QRADAR_CONF/$prefix.keystore -deststoretype jks -storepass estreamer 2>&1`;
		print "DEBUG: $output\n" if ($options{d});
		
		#Remove temporary pkcs12 created during repackaging.
		if (-e 'Client.pkcs12')
		{
			unlink('Client.pkcs12') || print "ERROR: Failed to remove temporary file [Client.pkcs12].\n";	
		}
		
		if ($output !~ /Import command completed:\s+1 entries successfully imported/)
		{
			exitWithMessage("ERROR: Problem creating keystore certificate.\n",1);
		}
		elsif (!-e "$QRADAR_CONF/$prefix.keystore")
		{
			exitWithMessage("ERROR: Failed to create keystore file [$QRADAR_CONF/$prefix.keystore]. Please ensure the current user has write access to [$QRADAR_CONF].",1);
		}
		else
		{
			print "Successfully generated keystore file [$QRADAR_CONF/$prefix.keystore].\n";
		}

		#===================================================#

	}
	else 
	{
		print "ERROR: Unable to locate Client key in Pem Extract:\n";
		print $pem_data;
		exitWithMessage("Aborting import",1);
	}


 	if ($options{d})
	{	
		print "\nContents of Truststore:\n";
		print "=======================\n";
		print `keytool -list -v -keystore $QRADAR_CONF/$PREFIX.truststore -storepass estreamer`;
		print "\nContents of Keystore:\n";
		print "=======================\n";
		print `keytool -list -v -keystore $QRADAR_CONF/$PREFIX.keystore -storepass estreamer`;
	}

	#And finally if there is a trusted certificates directory, copy your new keystore and truststore files into there	
	&copy_to_trusted_certificates("$QRADAR_CONF/$prefix.keystore");
	&copy_to_trusted_certificates("$QRADAR_CONF/$prefix.truststore");

}
